/*****************************************************************************************
* File name: DMC_Con.c
* Purpose:  Start the Connection the SPU to DEC/MEC.	
*
* Description:	This Program Acknowledges the Reception of the Connect to DEC/MEC Command 
*				and start the Link to DEC/MEC as Master or Slave depending on the Attached
*               Parameter.
*****************************************************************************************/

#ifdef SIMULATOR
#include "virtuosoSim.h"
#else // SIMULATOR
#include "node1.h"
#include "k_struct.h"
#endif

#include "spulib.h"
#include "spu_io.h"

/*=================================================================================================*/
/*External function and variables
===================================================================================================*/

extern u4 SpuIsSlaveToDmc, LinkConnectionTimeOut;

/*================================================================================================*/

/*
This function handles the 1355 connection to DMC.
*/
void PerformConnectToDmc(u4 buf[])
{
	struct FIFO_TRANSMIT_MSG fifoTransmitMsg;	//msg to communicate with the data tx task
	u4 sid=buf[1]&0xFFFF;
	
	/*=============================================================================================*/
	
	if (sid!=1) /* Negative Acknowledgment:  Invalid Structure ID*/
	{
		fifoTransmitMsg.Header = K_NACK_PERFORM | K_INVALID_SID;
	  	fifoTransmitMsg.Block1.NackParameter = sid;

		KS_FIFOPut(FIFO_TRANSMIT, &fifoTransmitMsg); /* Send the Response Packet to the DPU */
	}
	else if ((buf[2]!=0x11)&&(buf[2]!=0x22)) /* Invalid SID Parameter: Link Start-up Protocol*/
	{
		fifoTransmitMsg.Header = K_NACK_PERFORM | K_INVALID_SID_PARAMETER;
	  	fifoTransmitMsg.Block1.NackParameter = buf[2];

		KS_FIFOPut(FIFO_TRANSMIT, &fifoTransmitMsg); /* Send the Response Packet to the DPU */
	}
	else /*Positive Acknowledgment */		
	{
		/* Send the Positive Acknowledgment to DPU */
		fifoTransmitMsg.Header = K_PACK_PERFORM;
		KS_FIFOPut(FIFO_TRANSMIT, &fifoTransmitMsg);
		
		/* Start the Link to DEC/MEC*/ 
#ifndef SIMULATOR

		/* Startup protocol */
		if(buf[2]==0x11) /* Master */
		{
			SpuIsSlaveToDmc = K_NO;
		}
		else /* Slave */
		{
			SpuIsSlaveToDmc = K_YES;
		}
		
		/* Set the Link Connection Timeout (case of SPU Slave)
		=============================================================================================*/
		LinkConnectionTimeOut = K_DMC_LINK_CONNECTION_TIMEOUT;

		KS_TaskStart(DMCLINK);			/* DEC/MEC Link Start Task */
#endif
	}
}
