#define LINK1355_H "Link1355.cpp"

/*$Header: /CVS/pacs/obsw/spu/devel/source/SPU_Sim/link1355.h,v 1.1 2006/08/26 13:35:55 amazy Exp $*/


#ifdef CSL_SHOW_INCLUDE
#pragma message( "Including " __FILE__ ) 
#endif


/*
During the development process, we will use some function either
in the simulators (running on PC) or in the OnBoard application 
(running with virtuoso).  

So, we redefine the functions used to control the links 1355 in order
to use the same syntax in all situation even if the communication channel
we use is different.

According to the platform and the mode of operation, the two functions 
Link1355Read and Link1355Write will use :
- the Link 1355 of the mosaic board (when running with Virtuoso)
- the Link 1355 of the 4Link Board (when running on a PC and when connected 
	directly to a board)
- the TCP/IP protocol over the internet (when running on a PC and when connected
	remotly to a board).

*/

#ifdef WIN32 // if our application is a simulator (it is running under Windows)
/*********************************************************************
SIMULATORS 
*********************************************************************/

#include "stdlib.h"

typedef enum 
{
	TCP_IP,
	LINK1355,
	NOT_DEF
} ConnectionMode;

typedef enum
{
	COM_SERVER = 0x02, // = SERVER in "u_socket.h",
	COM_MASTER = COM_SERVER, 
	COM_CLIENT = 0x01, // = CLIENT in "u_socket.h",
	COM_SLAVE  = COM_CLIENT, 
} ConnectionType;

/*Link1355 may be replaced by Sockets.  
According to the value of gConnectionMode, the Link1355Read
and Link1355Write functions will use sockets or the link1355
on the 4 Link Board.
*/

#ifdef COMDLL_EXPORTS
#define COM_API __declspec(dllexport)
#else
#define COM_API __declspec(dllimport)
#endif

COM_API void OpenLinkTCP(int device, ConnectionType type, int port, const char* p_serverName);
COM_API void OpenLink1355(int device, ConnectionType type, int pciChannelNb);
COM_API void CloseLink(int device);

COM_API void Link1355Read(int device, int sizeInWords, void* pDataBlock);
COM_API void Link1355Write(int device, int sizeInWords, void* pDataBlock);
COM_API int Link1355Test(int device);
#else //WIN32    : our application is running under Virtuoso

/*********************************************************************
VIRTUOSO
*********************************************************************/

// redirect function call to RAW link drivers of virtuoso
#define Link1355Read(device, size, pDataBlock) KS_LinkinW(device/3+1, device%3+1, size, pDataBlock)
#define Link1355Write(device, size, pDataBlock) KS_LinkoutW(device/3+1, device%3+1, size, pDataBlock)
#define Link1355Test(device)	KS_RawLinkCOMITest(device/3+1, device%3+1)
#endif //WIN32


