/***************************************************************************************
* File name: Str_Stp.c
* Purpose:  Acknowledges the Start or Stop Command.
*
* Description:	This Program Acknowledges the Reception of the Start or the Stop 
*				Command and Performs the Appropriate Action in Case of Command 
*				Consistency.
****************************************************************************************/

#ifdef SIMULATOR
#include "virtuosoSim.h"
#else // SIMULATOR
#include "node1.h"
#include "k_struct.h"
#endif
#include "spuAsw.h" 
#include "spulib.h"


/*
This function handles the start/stop command
*/
void PerformStartStop(u4 buf[])
{
	struct FIFO_TRANSMIT_MSG fifoTransmitMsg;	//msg to communicate with the data tx task
	u4 sid = buf[1] & 0xFFFF;
	
	/*=============================================================================================*/

	if (sid!=0) /* Negative Acknowledgment:  Invalid Structure ID*/
	{
		fifoTransmitMsg.Header = K_NACK_PERFORM | K_INVALID_SID;
	  	fifoTransmitMsg.Block1.NackParameter = sid;
		
		KS_FIFOPut(FIFO_TRANSMIT, &fifoTransmitMsg); /* Send the Response Packet to the DPU */
	}
	else /* Positive Acknowledgment */		
	{
		fifoTransmitMsg.Header = K_PACK_PERFORM;
		
		KS_FIFOPut(FIFO_TRANSMIT, &fifoTransmitMsg); /* Send the Response Packet to the DPU */
	
		/* Case of the Start Command */
		if (buf[1]==0x80000)
		{
			/* To Signal the Start of the Compression Software*/
			CompressionSoftwareRunning = K_YES;

			/* Set HK for "Compression is started" */
			Result.CompStatus = K_COMP_STATUS_CSW_RUNNING;
		}
		else
		{		/* Case of the Stop Command */
			CompressionSoftwareRunning = K_NO;
			DataGeneratorRunning = K_NO;
			
			/* To Stop the Buffer Transmission Mode and Reset the Buffer */
			BufferTransmissionModeEnabled = K_NO;
			BufferTransmissionModeBufferId = 0;
			
			/* Set HK for "Compression is stopped" */
			Result.CompStatus = K_COMP_STATUS_CSW_STOPPED;
		}
	}
}
