/****************************************************************************************
* File name: datatx.c
* Purpose:  Sends Data to DPU.			 
*
* Description:	The DataTransmission Program Waits for the All Tasks Outputs and 
*				send them to The DPU. This Task Organizes the Data Flow from SPU
*				to DPU.
*****************************************************************************************/
 
#ifdef SIMULATOR
#include "virtuosoSim.h"
#include "stdio.h"
#else // SIMULATOR
#include "node1.h"
#include "k_struct.h"
#endif
#include "spulib.h"
#include "spuAsw.h"

/*=================================================================================================*/
/*Global variables declared in this file
===================================================================================================*/

u4 DumpBuffer[256];

/*=================================================================================================*/
/*External function and variables
===================================================================================================*/

#ifdef SIMULATOR
extern u4 CbufFreeSpaceSummary;
#endif

/*================================================================================================*/

/*
This is the entry point of the data tx task
*/
void DataTransmit()
{
	u4 bufferToSend[280];
	struct FIFO_TRANSMIT_MSG fifoTransmitMsg;
	
   	/*============================================================================================*/
	
	while (1)
	{
		/* Wait that Another Task Writes in the FIFO */
		KS_FIFOGetW(FIFO_TRANSMIT, &fifoTransmitMsg);
		
		/* Recognize the Packet Type (Response, HK or Science Data Packet) */
		bufferToSend[0] = fifoTransmitMsg.Header;
		
		switch(fifoTransmitMsg.Header&0xFFFF0000)
		{
		case 0x00870000: /* HK Packet */	  
			{		
				MemCopyDmToDm((u4*)&HkPacket, bufferToSend, 18);
				
				/* Send Packet to DPU */
#ifdef SIMULATOR
				printf("PIXB:%X \b CIB:%X DMC_ERROR:%X BUFFER:%X\n", HkPacket.PacketIndex, HkPacket.CounterInc, HkPacket.DmcHeaderError, CbufFreeSpaceSummary);
#endif
				Link1355Write(K_SMCSCHANNEL_1, 18, bufferToSend);
			}break;

		case (0x008A0000): /* Case of the Science Data Packet in Spectroscopy */
			{
				/* Transfer Sequentially TM Packets Whenever DPU is Ready to Receive */
				ReadInCircularBuffer(K_CBUF_TM, bufferToSend, 253);
				
				/* Send Packet to DPU */
				Link1355Write(K_SMCSCHANNEL_1, fifoTransmitMsg.Block1.Size, bufferToSend);
				fifoTransmitMsg.NbPacketsToTransmit--;
				
				if (fifoTransmitMsg.NbPacketsToTransmit != 0)
				{
					//if there are still packets to be transmitted, repost a message with the remaining number of packets
					//this should allow a HK packet to be transmitted inbetween science packets
					KS_FIFOPut(FIFO_TRANSMIT, &fifoTransmitMsg);
				}
			}break;
		case (0x008B0000): /* Case of the Science Data Packet in Photometry*/
			{
				/* Transfer Sequentially TM Packets Whenever DPU is Ready to Receive */
				ReadInCircularBuffer(K_CBUF_TM, bufferToSend, 253);
				
				/* Send Packet to DPU */
				Link1355Write(K_SMCSCHANNEL_1, fifoTransmitMsg.Block1.Size, bufferToSend);
				fifoTransmitMsg.NbPacketsToTransmit--;
				
				if (fifoTransmitMsg.NbPacketsToTransmit != 0)
				{
					//if there are still packets to be transmitted, repost a message with the remaining number of packets
					//this should allow a HK packet to be transmitted inbetween science packets
					KS_FIFOPut(FIFO_TRANSMIT, &fifoTransmitMsg);
				}
			}break;
		case K_PACK_LOAD: /* Case of the Positive Acknowledgment "Load"*/
			{
				/* Send Packet to DPU */
				Link1355Write(K_SMCSCHANNEL_1,1, bufferToSend); 
			}break;

		case K_NACK_LOAD:	/* Case of the Negative Acknowledgment "Load"*/
			{	
				bufferToSend[1] = fifoTransmitMsg.Block1.NackParameter;

				/* Send Packet to DPU */
				Link1355Write(K_SMCSCHANNEL_1,2, bufferToSend);	 
			}break;

		case K_PACK_DUMP_INTER: /* Case of the Positive Acknowledgment "Dump of the Intermediate 
								or First Packet"											 */
			{
				MemCopyDmToDm(DumpBuffer, bufferToSend,(fifoTransmitMsg.Block1.AddressLength&0xFFFF)+3);

				/* Send Packet to DPU */
				Link1355Write(K_SMCSCHANNEL_1,(fifoTransmitMsg.Block1.AddressLength&0xFFFF)+3, bufferToSend);
			}break;
	
		case K_PACK_DUMP_LAST:   /* Case of the Positive Acknowledgment "Dump of the LastPacket"*/
			{
				MemCopyDmToDm(DumpBuffer, bufferToSend,(fifoTransmitMsg.Block1.AddressLength&0xFFFF)+3);
				
				/* Send Packet to DPU */
				Link1355Write(K_SMCSCHANNEL_1,(fifoTransmitMsg.Block1.AddressLength&0xFFFF)+3, bufferToSend);
			}break;
	
		case K_NACK_DUMP:	  /* Case of the Negative Acknowledgment "Dump"*/
			{
				bufferToSend[1]=fifoTransmitMsg.Block1.NackParameter;
					
				/* Send Packet to DPU */
				Link1355Write(K_SMCSCHANNEL_1,2, bufferToSend);
			}break;
				
		case K_PACK_CHECK:   /* Case of the Positive Acknowledgment "Check"*/
			{
				bufferToSend[1]=fifoTransmitMsg.Block1.AddressLength;
				bufferToSend[2]=fifoTransmitMsg.Checksum;
				
				/* Send Packet to DPU */
				Link1355Write(K_SMCSCHANNEL_1,3, bufferToSend);
			}break;

		case K_NACK_CHECK:	/* Case of the Negative Acknowledgment "Check"*/
			{
				bufferToSend[1]=fifoTransmitMsg.Block1.NackParameter;
				/* Send Packet to DPU */
				Link1355Write(K_SMCSCHANNEL_1,2, bufferToSend);
			}break;
			
		case K_PACK_PERFORM:	 /* Case of the Positive Acknowledgment "Perform"*/
			{
				/* Send Packet to DPU */
			Link1355Write(K_SMCSCHANNEL_1,1, bufferToSend);
			}break;
	
		case K_NACK_PERFORM:  /* Case of the Negative Acknowledgment "Perform"*/
			{
				bufferToSend[1]=fifoTransmitMsg.Block1.NackParameter;
				
				/* Send Packet to DPU */
				Link1355Write(K_SMCSCHANNEL_1,2, bufferToSend);
			}break;

		case K_PACK_WRITE:	  /* Case of the Positive Acknowledgment "Write"*/
			{
				/* Send Packet to DPU */
				Link1355Write(K_SMCSCHANNEL_1,1, bufferToSend);	
			}break;
		
		case K_NACK_WRITE:	  /* Case of the Negative Acknowledgment "Write"*/
			{
				bufferToSend[1]=fifoTransmitMsg.Block1.NackParameter;
				
				/* Send Packet to DPU */
				Link1355Write(K_SMCSCHANNEL_1,2, bufferToSend);
			}break;

		case K_NACK_INVALID_CMD:  /* Case of the Negative Acknowledgment "Invalid Command"*/
			{
				bufferToSend[1]=fifoTransmitMsg.Block1.NackParameter;

				/* Send Packet to DPU */
				Link1355Write(K_SMCSCHANNEL_1,2, bufferToSend);
			}break;
		}
	}
}
