/***************************************************************************************
* File name: genspu.h.c
* Description:	This Header File Contains the Global Definitions and Macro-Functions
****************************************************************************************/

#ifndef _genspu_H
#define _genspu_H

/*===========================================================================================================*/
/* General Definitions 
=============================================================================================================*/

#define u4					unsigned int
#define K_YES				0
#define K_NO				1
#define K_SPU_HLSW_VERSION_ID 0x0D			/* Version ID of the SPU HLSW */
#define K_HLSW_BUILDNR		96				/* Sub version ID of SPU HLSW */
#define K_SMCSCHANNEL_1		1				/* DPU Channel: Link 1 */
#define K_SMCSCHANNEL_2		2				/* DEC/MEC Channel: Link 2*/

/*===========================================================================================================*/
/* MACROs Definitions
=============================================================================================================*/

#define MEM32_CLR_BIT(ptr,pos) (ptr[(pos)>>5] &= BitMask2[(pos)&0x1f]) /* Clear Bit MACRO */
#define MEM32_SET_BIT(ptr,pos) (ptr[(pos)>>5] |= BitMask1[(pos)&0x1f]) /* Set Bit to 1 MACRO */
#define MEM32_GET_BIT(ptr,pos) (ptr[(pos)>>5] & BitMask1[(pos)&0x1f])  /* Read Bit MACRO */
#define MEM32_GET_VBIT(value,pos) (value & BitMask1[pos])				/* Read Bit in a Value MACRO */
#define MEM32_PUT_BIT(ptr,pos,value) ((value & 0x01) ? MEM32_CLR_BIT(ptr,pos) : MEM32_SET_BIT(ptr,pos)) /* Write to BIT MACRO*/
#define MEM32_PUT_N_BITS(ptr,pos,nbits,value) {u4 ctr=0; for ( ; ctr<nbits; ctr++) (MEM32_GET_VBIT(value,32-nbits+ctr) ? MEM32_SET_BIT(ptr,pos+ctr) : MEM32_CLR_BIT(ptr,pos+ctr));} /* WRITE a Block Data to N Bits MACRO*/
#define LEFT_ZEROS_COUNT(input,result) asm("%0 =leftz %1;":"=d"(result):"d"(input))




/*===========================================================================================================*/
/* BIT_STREAM Definitions
=============================================================================================================*/

/*
How to use the BIT_STREAM functionality:
BIT_STREAM is simply a helper structure and set of functions to fill a buffer bit per bit or blocks per blocks.
The buffer in which the bit stream will be stored must have been declared previously.
The BIT_STREAM functionality takes care of counting the number of bits that have been used and positioning the data at the right place.
For this purpose, only 'push' functions are provided to add data at the end of the stream

So, consider that you have a 'u4* dest' buffer already declared.
You must declare the BIT_STREAM variable:
BIT_STREAM bitStream

And inialize it (assign it a buffer and reset it):
DeclareBitStream(bitStream, dest)

Then, you can use any of the functions provided:
PushBits(bitStream, value, nbBits); //to push a block of bits at the end of the stream
PushBitTrue(bitStream);				//to push a 1 bit at the end of the stream
PushBitFalse(bitStream);			//to push a 0 bit at the end of the stream

Then, once the bit stream has been filled, you can access the data throug bitStream.Buffer
and you can access the size (nb of bits in the bitStream) through bitStream.CurrentBitPos.

Be aware that, with the optimized version in assembly, there can be 'trash' in the last word in the bits that were not used by the bit stream.
*/

//declaration of low level functions (C and assembly)
#ifdef SIMULATOR
extern void putnbits32(u4 value, u4 offset, u4 nbits, u4 *dest); 
#else // SIMULATOR
extern void putnbits (u4 value, u4 offset, u4 nbits, u4 *dest);
extern void putnbitsOpt(u4 value, u4 offset, u4 nbits, u4 *dest);
#endif //SIMULATOR

typedef struct BIT_STREAM		
{
	u4* Buffer;							/* the buffer containing the bit stream */
	u4 CurrentBitPos;					/* absolute bit pos */
} BIT_STREAM;

//the functions
#define PushBitFalse(bitStream) {(bitStream).Buffer[(bitStream).CurrentBitPos >> 5] &= BitMask2[(bitStream).CurrentBitPos++ & 0x1F];};
#define PushBitTrue(bitStream) {(bitStream).Buffer[(bitStream).CurrentBitPos >> 5] |= BitMask1[(bitStream).CurrentBitPos++ & 0x1F];};
#define DeclareBitStream(bitStream, buffer) {(bitStream).Buffer = buffer; (bitStream).CurrentBitPos = 0;};
#define PadBitStreamWithZeroes(bitStream) {if ((bitStream.CurrentBitPos%32) > 0) {PushBits(bitStream, 0, 32 - (bitStream.CurrentBitPos%32));}}	//fills the end of the bitstream with zeroes until the end of the current word
#ifdef SIMULATOR
#define PushBits(bitStream, value, nbBits) {putnbits32(value, (bitStream).CurrentBitPos, nbBits, (bitStream).Buffer); (bitStream).CurrentBitPos += nbBits;};
#else //SIMULATOR
#define PushBits(bitStream, value, nbBits) {putnbitsOpt(value, (bitStream).CurrentBitPos, nbBits, (bitStream).Buffer); (bitStream).CurrentBitPos += nbBits;};
#endif

/*===========================================================================================================*/
/* Sorting Definitions
=============================================================================================================*/

#define NULLO    0

/*===========================================================================================================*/
 
#endif

