/*****************************************************************************
 *
 * Project name: 			Plank LFI REBA / Herschel PACS SPU
 * Product name: 			LLSW_DRV
 * Object name: 			l_memory
 * Filename: 				%M%
 * Language: 				C (ADSP-21020)
 * Compiler: 				G21K - r3.3
 * Company: 				CRISA
 * Author: 					Jaime Gomez/F. Torrero
 * Version: 				%I%
 * Creation date: 			06/Aug/2001
 * Last Modification date: 	%G%
 *
 * Description ===============================================================
 *
 * This module implements functions for memory access
 *
 * Change log ===============================================================
 *
 * | DATE 	  | NEW VERSION 	| AUTHOR 	  | 	REASON FOR CHANGE 	|
 * ===========================================================================
 *
 * 06/Aug/01 	1			 	 F.Torrero		Creation
 *
 * ****************************************************************************/ 


#ifndef L_MEMORY_H
#define L_MEMORY_H



/*****************************************************************************
   INCLUDES
*****************************************************************************/

#include "l_gendef.h"
#include "l_hwmap.h"




/*****************************************************************************
   PROVIDED CONSTANTS
*****************************************************************************/





/*****************************************************************************
   PROVIDED TYPES
*****************************************************************************/


/*****************************************************************************
   PROVIDED VARIABLES
*****************************************************************************/


/*****************************************************************************
   DECLARATION OF PROVIDED FUNCTIONS
*****************************************************************************/


T_VOID MEM_CopyDMToDM16(T_UNSIGNED_32 DMSourceStartAddress,
					  T_UNSIGNED_32 DM16DestinationStartAddress,
					  T_UNSIGNED_32 NumberOf16bitWords);

T_VOID MEM_CopyDM16ToDM(T_UNSIGNED_32 DM16SourceStartAddress,
					  T_UNSIGNED_32 DMDestinationStartAddress,
					  T_UNSIGNED_32 NumberOf16bitWords);

T_VOID MEM_CopyDMToPM(T_UNSIGNED_32 DMStartAddress,
					  T_UNSIGNED_32 PMStartAddress,
					  T_UNSIGNED_32 NumberOf16bitWords);

T_VOID MEM_CopyPMToDM(T_UNSIGNED_32 PMStartAddress,
					  T_UNSIGNED_32 DMStartAddress,
					  T_UNSIGNED_32 NumberOf16bitWords);

T_VOID MEM_CopyPMToPM(T_UNSIGNED_32 PMSourceStartAddress,
					  T_UNSIGNED_32 PMDestinationStartAddress,
					  T_UNSIGNED_32 NumberOf48bitWords);

T_VOID MEM_WriteBitInDMWord(T_UNSIGNED_32 DMAddress,
							T_UNSIGNED_32 BitPosition,
							T_UNSIGNED_32 BitValue);

T_VOID MEM_WriteBitInPM32bitMSWord(T_UNSIGNED_32 PMAddress,
								   T_UNSIGNED_32 BitPosition,
								   T_UNSIGNED_32 BitValue);

T_BOOLEAN MEM_CheckBitInDMWord(T_UNSIGNED_32 DMAddress,
							   T_UNSIGNED_32 BitPosition);

T_BOOLEAN MEM_CheckBitInPM32bitMSWord(T_UNSIGNED_32 PMAddress,
									  T_UNSIGNED_32 BitPosition);

T_SR MEM_IdentifyPRAMOrEEPROMAddress(T_UNSIGNED_32 HardwareUnitID,
									 T_UNSIGNED_32 PMAddress,
									 T_UNSIGNED_32 *MemoryAreaIDPRAMOrEEPROM);

T_UNSIGNED_32 MEM_ReadDM16Word(T_UNSIGNED_32 DMAddress);


T_UNSIGNED_32 MEM_ReadDMWord(T_UNSIGNED_32 DMAddress);

T_UNSIGNED_32 MEM_ReadPM32bitMSWord(T_UNSIGNED_32 PMAddress);

T_UNSIGNED_16 MEM_ReadPM16bitLSWord(T_UNSIGNED_32 PMAddress);

T_VOID MEM_WriteDMWord(T_UNSIGNED_32 DMAddress,
					   T_UNSIGNED_32 DMWord);

T_VOID MEM_WritePMWord(T_UNSIGNED_32 PMAddress,
					   T_UNSIGNED_32 PMWord32MSbits,
					   T_UNSIGNED_16 PMWord16LSbits);




#endif  /* L_MEMORY_H */

