/***************************************************************************************
* File name: genspu.h.c
* Version.Revision: HPSPU-HLSW-OB-V12R0
*
* Public Functions: None
*
*
* Description:	This Header File Contains the Global Definitions and Macro-Functions
*
* Short Description: See Description
*
* Creation date & author: 19.07.2002 by Ahmed Nabil Belbachir
* Version, Update date & author: HPSPU-HLSW-OB-V12R0 19.09.2005 by Ahmed Nabil Belbachir
****************************************************************************************/

/* Declaration of the Header File 
===================================================================================================*/

#ifndef _pacscod_H
#define _pacscod_H

/*===========================================================================================================*/
/* General Definitions 
=============================================================================================================*/

#include "genspu.h"
#include "qsmodel.h"
#include "bitmodel.h"
#include "rangecod.h"

#define ALPHABETSIZE 256
#define CACHESIZE 32
#define MTFSIZE 20
#define MTFHISTSIZE 4096  /* must pe power of 2 */
#define MTFHISTSIZE1 4095  

typedef struct cacheS *cacheptr;
typedef struct cacheS {
    u4 symbol, sy_f, weight, what;
    cacheptr next, prev;
} cacheentry;

typedef struct {
    u4 sym, next;
} mtfentry;

typedef struct {
    u4 whatmod[3];  /* probabilities for the submodels */
    cacheptr newest,  /* points to newest element in cache */
             lastnew; /* points to last element with higher weight */
    u4 cachetotf;   /* total frequency count in cache */
    u4 mtffirst;    /* where to find the newest entry in mtfhist */
    u4 mtfsize;     /* size of mtflist */
    u4 mtfsizeact;  /* size of active mtflist */
    cacheptr lastseen[ALPHABETSIZE]; /* tell if and where symbol is in cache */
    cacheentry cache[CACHESIZE]; /* cache */
    mtfentry mtfhist[MTFHISTSIZE];
    bitmodel full;    /* fallback model */
    qsmodel mtfmod;   /* probabilities for mtf ranks */
    qsmodel rlemod[5];
    rangecoder ac;
} pacs_model;

#define initmodel(m,a,b) M_initmodel(a,b)
#define fixafterfirst(m) M_fixafterfirst()
#define deletemodel(m) M_deletemodel()
#define pacs_encode(m,a,b) M_pacs_encode(a,b)

void initmodel(pacs_model *m, int headersize, u4 *first);

/* call fixafterfirst after encoding the first run */
void fixafterfirst(pacs_model *m);

/* deletion of the model */
void deletemodel(pacs_model *m);

/* encode a run of equal symbols */
void pacs_encode(pacs_model *m, u4 symbol, u4 runlength);


/*===========================================================================================================*/
 
#endif

