/***************************************************************************************
* File name: phot_gn.c
* Purpose:  Photometric Data Generator.
*
* Description:	This Program Generates Simulated Data for Photometry.
****************************************************************************************/

#ifdef SIMULATOR
#include "virtuosoSim.h"
#else // SIMULATOR
#include "node1.h"
#include "k_struct.h"
#endif

#include "spulib.h"
#include "spuAsw.h"

/*=================================================================================================*/
/*Global variables declared in this file
===================================================================================================*/

struct SCIENCE_PACKET TestSciencePacket;

/*=================================================================================================*/
/*External function and variables
===================================================================================================*/

//these variables are used to simulate the reception of packets on the 1355 link
extern int ChannelReceiveAddress[3];
extern int ChannelReceiveSema[3];

/*=================================================================================================*/

/*
This is the entry point for the photo generator task
*/
void PhotoGenerator()
{
	u4 i, j, k, scienceData, readoutCounter;
	u4 chopperSlew;
    u4 plateau, nbBlocksPerDetector;
	
	/*=============================================================================================*/

	while(1)
	{
	    /* Wait For Command from DPU To Start Data Generation for the Test Mode */
		KS_SemaReset(SEMA_SPUTESTP);
		KS_SemaTestW(SEMA_SPUTESTP);
		
		/*Initilaization of the Parameters for the Test */
		scienceData = 0x01000100;
		readoutCounter = 0;
		chopperSlew = 0;
		nbBlocksPerDetector = 4;

	    if(SimulationDataParameters.SpuId != 2) 
		{
			nbBlocksPerDetector =1;
		}
	
		/*=============================================================================================*/
		
		
		while(DataGeneratorRunning==K_YES)   /* Generate Data for Two Chopper Plateaus */
		{
			TestSciencePacket.Block2.CurrentReadoutCountInChopperPlateau = 0;
			plateau = 0;
			chopperSlew = 0;
			while(plateau<3)				/* Generate Data for Two Chopper Plateaus */
			{
				plateau++;
				i=0;
				while((i<SimulationDataParameters.ReadoutsPerRamps)&&(DataGeneratorRunning==K_YES))
				{
					/* Prepares the DEC/MEC Header */
					TestSciencePacket.SpuId    = SimulationDataParameters.SpuId;	/* SPUID */
					TestSciencePacket.Type     = K_PHOTO;/* Observation Type */
				   	TestSciencePacket.ObsId    = 0x00;	/* OBSID */
				   	TestSciencePacket.BbId     = 0x00;	/* BBID */
					TestSciencePacket.Label    = 0x00;	/* LABEL */ 
					TestSciencePacket.Time[0]  = 0;		/* 1st Word of Time */
					TestSciencePacket.Time[1]  = 0;		/* 2nd Word of Time */
					TestSciencePacket.Validity = 0xFF;	/* Validity */
					TestSciencePacket.ChopperPos = 0x1000; /* CPR */
					TestSciencePacket.WheelPos   = 0x00;	/* WPR */
					TestSciencePacket.Block1.BolcStatus = 0;			/* Status of BolC */
					TestSciencePacket.CurrentReadoutCounter = readoutCounter++;		/* Current Readout Count Since the Last Time Reset */
					TestSciencePacket.Block2.CurrentReadoutCountInChopperPlateau++;				/* CRDCCP */
					TestSciencePacket.Block3.DataBlockId = 0;				/* DBID */
					TestSciencePacket.Block4.BolSetupId = 0;				/* BSID */
					TestSciencePacket.CompressionMode = SimulationDataParameters.CompressionMode;	/* CMM */
			
					/* Generates the Science Data*/
					for(k=0;k < K_NB_PIXELS_IN_PHOTO_ARRAY;k++)
					{
						TestSciencePacket.Data[k] = scienceData + chopperSlew;
					}
				    j=0;
					while((j<nbBlocksPerDetector)&&(DataGeneratorRunning==K_YES)) /* Send All Four Sub-Regions Sequentially*/
					{
				 		if(nbBlocksPerDetector==4) 
						{
							TestSciencePacket.Block3.DataBlockId= j+1;				/* Current Data Block ID for Blue */
						}
			            else 
						{
							TestSciencePacket.Block3.DataBlockId= 5;				/* Data Block ID for Red */
							KS_TaskSleep(15);
						}
						j++;
						
						/* Send the Data Packet to the Supervisor */
#ifndef SIMULATOR
						//simulate the reception of this packet on the 1335 link
						//copy the packet in DPRAM
						MemCopyDmToDm((u4*)&TestSciencePacket, (u4*)ChannelReceiveAddress[K_SMCSCHANNEL_2-1], 272);
						//and signal the semaphore
						KS_SemaSignal(ChannelReceiveSema[K_SMCSCHANNEL_2-1]);
#else
						KS_SemaSignal(19);  //this unused semaphore number is used only for the simulator on the PC
#endif
						KS_TaskSleep(5); /* Data Generated at 40 Hz*/	
					}
					i++;
					KS_TaskSleep(10); /* Data are Generated at 40 Hz */
				}
				chopperSlew = 0x01000100; /* Chopper Motion */
			}
		}
	}
}
