/****************************************************************************************
* File name: pkup_pg.c
* Version.Revision: HPSPU-HLSW-OB-V12R0
*
* Purpose:  Performs the Reduction Using Modelled Ramps*
* Public Functions: None
*
* Private Functions: None
*
* Description:	The Modelled Ramps Reduction Performs Lossless Compression of the
*               Spectroscopy Data.
*
* Creation date & author: 28.03.2001 by  Ahmed Nabil Belbachir
* Version, Update date & author: HPSPU-HLSW-OB-V12R0 30.08.2004 by Ahmed Nabil Belbachir
****************************************************************************************/

/* Declaration of The Header Files 
==================================================================================================*/

#ifdef SIMULATOR
#include "virtuosoSim.h"
#else // SIMULATOR
#include "node1.h"
#include "k_struct.h"
#endif
#include "spuAsw.h"
#include "spulib.h"	

/*================================================================================================*/
/* Declaration of The Used Functions
==================================================================================================*/

void Difference(u4 frames);
void Calcul_Model(u4 frames);
void noise_resampling();
extern void Llc_Spec();
extern u4 rc_reorder(u4 *src, u4 Size);

/*================================================================================================*/
/*Declaration of the Used Global Variables
==================================================================================================*/

u4 Model[0x1200];
u4 first_buffer = K_YES;  /* For Modelled Ramps*/
extern u4 Comp_DMCH_Buff[0x1500];
u4 Actual_ReadoutsPerRamps=0;
/*================================================================================================*/
/* The Main Program 
==================================================================================================*/

void pkup_pg()
{
	/*No Raw Channel Data are Sent in this Mode */
	RawChannelSelectionSpectro.NbSelectedRawChannels = 0;
	RawChannelSelectionSpectro.IndexOfFirstChannel = 0;
	

	/*=============================================================================================*/
	/* Reset of the HK Parameters of Compression Information. These HK are Only used While Reduction
	===============================================================================================*/

	Result.NbMaintainedRamps = 0;
	Result.SaturationFlag=0;
	Result.GlitchCounter = 0;
	Result.NbIntegratedRamps = 0;


	/* Check if Calculation of Averaged Ramps Or Start Compression
	===============================================================================================*/
    
	/*=============================================================================================*/
	/* DMC Header Extraction and Compression */
	DmcHeaderCompression(NbFramesInBuffer, K_PHOTO_HEADER_SIZE, mode);

	/* Detector Selection */
	if(CompressionSoftwareRunning==K_YES)
	 	Dxs(ScienceBufferSizeInWords, &DxsSpectro, NbFramesInBuffer, K_CBUF_SCIENCE_SPECTRO, 0, mode);
		
	/* Calculate Model */
	Calcul_Model(NbFramesInBuffer);
	
	/* Write the Compressed Data in the Output Buffer */
	fill_out(K_SPECTRO, Mode.PacketIndex, 0x20, Mode.ObsId);
}

/* End of the Main Program 
==================================================================================================*/
/********************************************************************************************
* Function name: noise_resampling
*
* Purpose: Lossless Compression of Spectroscopy Data 
*
* Description:	This Function Calculates the Difference between the modelled ramp and the 
*				Actual Ramp and then, compress this Difference Lossless.
*
* Syntax: void noise_resampling(void)
*
* Input: Address and Size of Data Buffer.
*
* Output: The Results are Re-stored in the source
*
* Return: None
*
* Files Accessed: None
*
* Creation date & author: 01.03.2004 by Ahmed Nabil Belbachir and Roland Ottensamer
*
* Version, Update date & author: HPSPU-HLSW-OB-V12R0 30.08.2004 by Ahmed Nabil Belbachir
*
********************************************************************************************/

/* Beging of the Noise Resampling Program 
==================================================================================================*/
void noise_resampling()
{

	/*No Raw Channel Data are Sent in this Mode */
	RawChannelSelectionSpectro.NbSelectedRawChannels = 0;
	RawChannelSelectionSpectro.IndexOfFirstChannel = 0;

	/*=============================================================================================*/
	/* Reset of the HK Parameters of Compression Information. These HK are Only used While Reduction
	===============================================================================================*/

	Result.NbMaintainedRamps = 0;
	Result.SaturationFlag=0;
	Result.GlitchCounter = 0;
	Result.NbIntegratedRamps = 0;

	/*=============================================================================================*/
	/* DMC Header Extraction and Compression */
	DmcHeaderCompression(NbFramesInBuffer, mode);
	
	/* Detector Selection */
	if(CompressionSoftwareRunning==K_YES)
		Dxs(ScienceBufferSizeInWords, &DxsSpectro, NbFramesInBuffer, K_CBUF_SCIENCE_SPECTRO, 0, mode);

	/* Temporal Redundancy Reduction */
	if(Mode.ReadoutsPerRamps!=Actual_ReadoutsPerRamps)
	{
		Mode.ReadoutsPerRamps=Actual_ReadoutsPerRamps;
		Difference(NbFramesInBuffer);
	}
	else
		Difference(NbFramesInBuffer);
			
	/* Data Sorting and Reordering Using the Position of a Sample in a Ramp */
	rc_reorder(ProcBuff+1, ProcBuff[0]);

	/* Lossless Coding */
	if(CompressionSoftwareRunning==K_YES)
		Llc_Spec();

	/* Write the Compressed Data in the Output Buffer */
	fill_out(K_SPECTRO, Mode.PacketIndex, K_CM_SPECTRO_NOISE_RESAMPLING, Mode.ObsId);
}

/* End of the Noise Resampling Program 
==================================================================================================*/

/* Begin of the Calcul_Model Program 
==================================================================================================*/

void Calcul_Model(u4 frames)
{
	u4 i,j, k, *src1, *src2, Nb_of_det_ramp=25*frames, Nb_elements=18*Mode.ReadoutsPerRamps;
	src1=ProcBuff+1;
	src2= Model;
	
	if(first_buffer ==K_YES)
	{
		/*=============================================================================================*/
		/* Clean the Already-stored Model*/
         for(i=0;i<0x1200;i++)
			 src2[i]=0;
	}

	if (Mode.ReadoutsPerRamps<257)
	{
		for (i=0;i<18;i++) /* Number of Rows */
		{ 
			for (j=0;j<Mode.ReadoutsPerRamps;j++)
			{
				for (k=0;k<Nb_of_det_ramp;k+=Mode.ReadoutsPerRamps)
				{
					src2[(Mode.ReadoutsPerRamps*i)+j]+= src1[k+(i*Nb_of_det_ramp)+j];
				}
			}
		}
	}
	if(first_buffer ==K_YES)
	{
		/* Next Time, Compression will start */
		first_buffer=K_NO;

		for(i=0;i<Nb_elements;i++)
		{
			src2[i] = src2[i]/Nb_of_det_ramp;
			src1[i] = src2[i];
		}
	}
	else
	{
		Nb_of_det_ramp++;
		for(i=0;i<Nb_elements;i++)
		{
			src2[i] = src2[i]/Nb_of_det_ramp;
			src1[i] = src2[i];
		}
	}
	ProcBuff[0]=Nb_elements;
	RawChannels.SizeInWords = 0;
	Comp_DMCH_Buff[0]=0;
	Actual_ReadoutsPerRamps= Mode.ReadoutsPerRamps;
}
/* End of the Calcul_Model Program 
==================================================================================================*/
/* Begin of the Difference Program 
==================================================================================================*/

void Difference(u4 frames)
{
	u4 i,j, k, Nb_of_det_ramp=(25*frames)/Mode.ReadoutsPerRamps;
	u4 *src1,*src2;
	src1=ProcBuff+1;
	src2= Model;

	for (i=0;i<18;i++) /* Number of Rows */
	{ 
		for (j=0;j<Mode.ReadoutsPerRamps;j++)
		{
			for (k=0;k<Nb_of_det_ramp;k+=Mode.ReadoutsPerRamps)
			{
				src1[k+(i*Nb_of_det_ramp)+j]=src1[k+(i*Nb_of_det_ramp)+j]-src2[(Mode.ReadoutsPerRamps*i)+j];
			}
		}
	}
}
/* End of the Difference Program 
==================================================================================================*/