/***************************************************************************************
* File name: qsmodel.h
* Version.Revision: HPSPU-HLSW-OB-V12R0
*
* Purpose:  Header for the quick sort model.
*
* Public Functions: None 
*
* Private Functions: None
*
* Description:	This file contains the declared parameters for the QS model.
*
* Creation date & author: 28.10.2004 by  Ahmed Nabil Belbachir
* Version, Update date & author: HPSPU-HLSW-OB-V12R1 18.01.2006 by Ahmed Nabil Belbachir
****************************************************************************************/

/* The Header Files 
===================================================================================================*/

#ifndef QSMODEL_H
#define QSMODEL_H

/*
  qsmodel.h     headerfile for quasistatic probability model

 
*/

#include "spuAsw.h"

typedef struct {
    int n;             /* number of symbols */
    int  left;          /* symbols to next rescale */
    int  nextleft;      /* symbols with other increment */
    int  rescale;       /* intervals between rescales */
    int  targetrescale; /* should be interval between rescales */
    int  incr;          /* increment per update */
    int  searchshift;   /* shift for lt_freq before using as index */
    u4 *cf;         /* array of cumulative frequencies */
    u4 *newf;         /* array for collecting ststistics */
    u4 *search;       /* structure for searching on decompression */
} qsmodel;

/* initialisation of qsmodel                           */
/* m   qsmodel to be initialized                       */
/* n   number of symbols in that model                 */
/* lg_totf  base2 log of total frequency count         */
/* rescale  desired rescaling interval, should be < 1<<(lg_totf+1) */
/* init  array of int's to be used for initialisation (NULLO ok) */
/* compress  set to 1 on compression, 0 on decompression */
void initqsmodel( qsmodel *m, int n, int lg_totf, int rescale,int *init);
void initqsmodel1( qsmodel *m, int n, int lg_totf, int rescale,int *init, int i );

/* reinitialisation of qsmodel                         */
/* m   qsmodel to be initialized                       */
/* init  array of int's to be used for initialisation (NULLO ok) */
void resetqsmodel( qsmodel *m, int *init);

/* retrieval of estimated frequencies for a symbol     */
/* m   qsmodel to be questioned                        */
/* sym  symbol for which data is desired; must be <n   */
/* sy_f frequency of that symbol                       */
/* lt_f frequency of all smaller symbols together      */
/* the total frequency is 1<<lg_totf                   */
void qsgetfreq( qsmodel *m, int sym, int sym1, int *sy_f, int *lt_f );


/* update model                                        */
/* m   qsmodel to be updated                           */
/* sym  symbol that occurred (must be <n from init)    */
void qsupdate( qsmodel *m, int sym );

#endif
