/**************************************************************************************
* File name: Spec3.c
* Purpose: Performs Data Compression for the 3rd Spectroscopic Mode.
*
* Description:	This Program Performs Data Compression in Spectroscopy for the
*				SpectroLosslessCompression Mode According to the Received Compression Parameter.
****************************************************************************************/

#ifdef SIMULATOR
#include "virtuosoSim.h"
#else // SIMULATOR
#include "node1.h"
#include "k_struct.h"
#endif
#include "spuAsw.h"
#include "spulib.h"	

/*=================================================================================================*/
/*External function and variables
===================================================================================================*/

extern void LosslessCompressionForSpectro(COMPRESSION_MODE mode);


/*
in lossless compression mode, there is no data reduction, only compression.
There is a limited number of selected pixels
*/
void SpectroLosslessCompressionMode(u4 nbFrames, COMPRESSION_MODE mode)
{
	/*=============================================================================================*/
	/* Reset of the HK Parameters of Compression Information. These HK are Only used While Reduction
	===============================================================================================*/

	Result.NbMaintainedRamps = 0;
	Result.SaturationFlag=0;
	Result.GlitchCounter = 0;
	Result.NbIntegratedRamps = 0;

	/*=============================================================================================*/
	/* DMC Header Extraction and Compression */
	DmcHeaderCompression(nbFrames, K_SPECTRO_HEADER_SIZE, mode);
   
	/* Detector Selection */
	Dxs(nbFrames * K_SPECTRO_FRAME_SIZE_IN_WORDS, &DxsSpectro, nbFrames, K_CBUF_SCIENCE_SPECTRO, 0, mode);

	/* Data Sorting and Reordering Using the Position of a Sample in a Ramp */
	TSReductionForSpectro(mode);

	if (CompressionSoftwareRunning == K_YES) //check that the compression software has not been stopped in the meanwhile
	{
		/* Lossless Coding */
		LosslessCompressionForSpectro(mode);

		/* Write the Compressed Data in the Output Buffer */
		BuildCompressedEntity(mode);
	}
}
