/****************************************************************************************
* File name: spec_gn.c
* Purpose:  Spectroscopic Data Generator.
*
* Description:	This Program Generates Simulated Data for Spectroscopy.
****************************************************************************************/

#ifdef SIMULATOR
#include "virtuosoSim.h"
#else // SIMULATOR
#include "node1.h"
#include "k_struct.h"
#endif

#include "spulib.h"
#include "spuAsw.h"

/*=================================================================================================*/
/*External function and variables
===================================================================================================*/

//these variables are used to simulate the reception of packets on the 1355 link
extern int ChannelReceiveAddress[3];
extern int ChannelReceiveSema[3];
extern struct SCIENCE_PACKET TestSciencePacket;


/*
This is the entry point for the spectro generator task
*/
void SpectroGenerator()
{
	u4 i,j,k;
    u4 nbRampsIn2secBuffer = 8;

	/*=============================================================================================*/

	while(1)
	{
	    /* Wait For Command from DPU To Start Data Generation for the Test Mode */
		KS_SemaReset(SEMA_SPUTESTS);
	    KS_SemaTestW(SEMA_SPUTESTS);

		/*Initilaization of the Parameters for the Test */
		TestSciencePacket.Block3.CurrentRampCounter = 0;

		
		/*=============================================================================================*/
		/* Set Default Parameters if the Table is not Available*/
		
		/* Define Number of Ramps in 2s*/
		if (SimulationDataParameters.ReadoutsPerRamps==1024)
			nbRampsIn2secBuffer = 1;
		else if (SimulationDataParameters.ReadoutsPerRamps!=0)
			nbRampsIn2secBuffer = roldiv(512, SimulationDataParameters.ReadoutsPerRamps);

		/* Start Generating Data*/
		/*Check the Interruption from the Watch Process "Stop Command"*/
		while(DataGeneratorRunning==K_YES) 
		{
			i=0;
			while((i<nbRampsIn2secBuffer) && (DataGeneratorRunning==K_YES)) /* to generate a 2-Second testSciencePacket*/
			{
				j=0;
				TestSciencePacket.Block3.CurrentRampCounter++; /* Increment the Ramp Counter */
				while((j<SimulationDataParameters.ReadoutsPerRamps)&&(DataGeneratorRunning==K_YES))
				{
					/* Prepares the DEC/MEC Header */
					TestSciencePacket.SpuId    = SimulationDataParameters.SpuId;	/* SPUID */
					TestSciencePacket.Type     = K_SPECTRO;/* Observation Type */
				   	TestSciencePacket.ObsId    = 0x00;	/* OBSID*/
				   	TestSciencePacket.BbId     = 0x00;	/* BBID*/
					TestSciencePacket.Label    = 0x00;	/* LABEL */
					TestSciencePacket.Time[0]  = 0;		/* 1st Word of Time */
					TestSciencePacket.Time[1]  = 0;		/* 2nd Word of Time */
					TestSciencePacket.Validity = 0xFF;	/* Validity 0xFF for Valid Data */
					TestSciencePacket.ChopperPos = 0x1000;	/* CPR */
					TestSciencePacket.WheelPos   = 0x00;	/* WPR */
					TestSciencePacket.Block1.GratingPos = 0x1000;	/* GPR */
			   		TestSciencePacket.CurrentReadoutCounter   = SimulationDataParameters.ReadoutsPerRamps-j-1;		/* Current Readout Count in a Ramp */
			   		TestSciencePacket.Block2.ReadoutsPerRamps = SimulationDataParameters.ReadoutsPerRamps;			/* RRR Readout in Ramp Readback */
					TestSciencePacket.Block4.CreControlReadback = 0;				/* CRECR */
			   		TestSciencePacket.CompressionMode = SimulationDataParameters.CompressionMode;	/* Compression Parameter */
		
					/* Generates the Science Data*/
					for(k=0;k< K_SPECTRO_FRAME_SIZE_IN_WORDS;k++)
						TestSciencePacket.Data[k] = 0x01000100 + (((SimulationDataParameters.ReadoutsPerRamps-j)<<16)|(SimulationDataParameters.ReadoutsPerRamps-j));

					/* Send the Data Packet to the Supervisor */
#ifndef SIMULATOR
					//simulate the reception of this packet on the 1335 link
					//copy the packet in DPRAM
					MemCopyDmToDm((u4*)&TestSciencePacket, (u4*)ChannelReceiveAddress[K_SMCSCHANNEL_2-1], 250);
					//and signal the semaphore
					KS_SemaSignal(ChannelReceiveSema[K_SMCSCHANNEL_2-1]);
#else
					KS_SemaSignal(19);  //this unused semaphore number is used only for the simulator on the PC
#endif
					KS_TaskSleep(4); /* Data Generated at 250 Hz on DSP*/	

					j++;
				}
				i++;
			}
		}
	}
}
